# Dr. Higgins v0.95v
# Intelligent Vocabulary Quiz 
# to help you with your self-study of a foreign language
# 
# Copyright (C) 2004-2005 Martijn van Iersel 
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
#################
###   Quiz   ###
#################

package Quiz;

use strict;
use warnings;

sub new { # 
    my $class  = shift;        
    my $self   = { };                           

    $self->{counter} = 0;
    $self->{questions} = undef;
    $self->{filename} = undef;
    
    $self->{question1} = 'What is ""?';
    $self->{question2} = 'What is ""?';
    $self->{bins} = 4;
    $self->{askbothways} = 1;
    $self->{charset} = "iso-latin-1";

    $self->{duration} = 0;
    $self->{sessions} = 0;
    
    return(bless($self, $class));
}

# reset and load nouns from file
    
sub reload { # $filename
    my $self = shift;
    my $filename = shift;
    
    open (INPUTFILE, "$filename") or die "Cannot open file \"$filename\"\n\n";    
    my @nouns;
    {
	local $/; #enable slurp mode in this block
	my $temp = <INPUTFILE>; #slurp entire file
	$temp =~ s/\015?\012/\n/g; #convert dos or unix line-ends
	$temp =~ s/\015/\n/g; # convert mac line-ends
	@nouns = split /\n/, $temp;
    }
    
    #chomp @nouns;
    close INPUTFILE;
    
    my @nounObj;
    
    my %q;
    my %a;
    foreach my $noun (@nouns)
    {
	#skip empties
	if ($noun eq "") { next; } 
	#check for comment lines
	if ($noun =~ /^#/)
	{
	    my @possible_tags = 
	       qw (question1 question2 askbothways bins charset);
	    foreach my $tag (@possible_tags)
	    {
		if ($noun =~ /^#($tag)=(.*)/)
		{
		    $self->{$tag} = $2;
		}
	    }
	    next; # skip comment lines
	}
	#$noun =~ tr//\x{94}\x{84}/;    	    
	unless ($noun =~ /^(.*), (.*)/)
	{
	    print "Consistency check failed, ",
		"syntax of line \"$noun\" is incorrect\n";
	    print "Ignoring line...\n";
	    next;
	}
	$noun =~ /^(.*), (.*)/;    	    
	my ($eng, $fin) = ($1, $2);
	   if (defined $a{$eng})
	   {
	       print "Consistency check failed, two questions exist that",
		  "both have the answer $eng\n";
	       print "Ignoring line...\n";
	       next;
	   }
	   if (defined $q{$fin})
	   {
	       print "Consistency check failed, two questions exist that",
		  "both have the answer $fin\n";
	       print "Ignoring line...\n";
	       next;
	   }
	   $q{$fin} = 1;
	   $a{$eng} = 1;
	push (@nounObj, NounObj->new($eng, $fin, 0));
	push (@nounObj, NounObj->new($fin, $eng, 1));
    }

    $self->{counter} = 0;
    $self->{questions} = \@nounObj;
    $self->{filename} = $filename;
}

sub printReport ()
{
    my $self = shift;

    print "Finished!\n\n";
		    
    print "Top 10 most difficult words:\n";
		    
    # pick the ones where quizCount is highest
    my @temp = @{$self->{questions}};
    sub byQuizCount { $b->{quizCount} <=> $a->{quizCount} }
    @temp = sort byQuizCount @temp;
		    
    my $outfile = $self->{filename};
    $outfile =~ s/\.txt$/_ten_most_difficult\.txt/;
    open (OUTFILE, "> $outfile");
		    
    foreach (@temp[0..9])
    {
	    my $percentage;
	    if ($_->{quizCount} > 0) 
	    { 
		$percentage = $_->{correctCount} * 100 / $_->{quizCount};   
	    }
	    else { $percentage = 0; }
	    print $_->{question}, " = ", $_->{answer}, "    - ";
	    printf "%3.2f%% correct \n", $percentage;
			    
	    print OUTFILE $_->{question}, ", ", $_->{answer}, "\n";
	    my $i;
	    for $i (@${$_->{wronganswer}})
	    {
		    print " " x 10 , $i, "\n";
	    }
    }
    close (OUTFILE);
    print "Words written to $outfile\n";
    print "\nPress Enter\n";
    <STDIN>;
}

sub run  #console, debugmode
{
    my $self = shift;
    my $con = shift;
    my $debugMode = shift;
    
    $con->Cls(0x17);
    for (;;)
    {
	    $con->Cursor (0,4);
    
	    my $noun;
	    $noun = $self->pickNext ();    
	    unless ($noun) { 
	    

		    $con->Cls(0x17);
		    $self->printReport();
		    return ;
	    }

	    my %box;
	    for (1..$self->{bins}) { $box{$_} = 0; }
	    my $dueCount = 0;
	    foreach $noun (@{$self->{questions}})
	    {
		#if noun happened to get past last box,
		#count it in last box
		if ($noun->{box} > $self->{bins})
		{
		   $box{$self->{bins}}++;
		}
		else
		{
		   $box{$noun->{box}}++;
		}
		if ($noun->{howSoon} != -1) { $dueCount++; }
	    }

	    foreach (1 .. $self->{bins})
	    {
		print " " x 10 . "Bin $_: ";
		$con->Attr(0xE7);
		my $frac = int ($box{$_} * 40 / scalar @{$self->{questions}});
		print "*" x $frac;
		$con->Attr(0x17);
		print " " x (40 - $frac);
		print " ($box{$_})\n";
		
	    }
	    print "$dueCount due for repetition\n" if ($debugMode);
	    $con->Cursor (0, 20);
	    print "    Question " . $self->{counter} . ": \n";
	    $con->Attr(0x1F);		    
	    my $question = ($noun->{dir} ? 
		    $self->{question1} : $self->{question2});
	    $question =~ s/""/"$noun->{question}"/;
	    print "    $question\n";
	    $con->Attr(0x17);		
	    print "    (answer \"Q\" to return to main menu)\n";
	    print "    ";

	    my $input = <STDIN>;
	    chomp $input;
   $input =~ s/"o//g;    	    
   $input =~ s/"a//g;
   $input =~ s/"e//g;
   $input =~ s/~n//g;
   $input =~ s/"A//g;
   $input =~ s/^\\//;
   $input =~ s/\\$//;    	    
   $input =~ s/'e//g;
   $input =~ s/'i//g;
   $input =~ s/'a//g;
   $input =~ s/'o//g;
   $input =~ s/'u//g;
   $input =~ s/\?(\w)/$1/g;
   $input =~ s/\!(\w)/$1/g;
    $con->Cls(0x17);
	$con->Cursor (0,10);
	print " " x 20 . "You answered: \"$input\"\n";
	if ($input eq 'Q' || $input eq 'q')
	{
	    return;
	}
	else
	{
	    if ($noun->compareAnswer($input, $self->{counter}++))
	    {
		    $con->Attr(0x1A);
		    print " " x 20 . "Correct!\n";
		    $con->Attr(0x17);
	    }
	    else
	    {
		    $con->Attr(0x1C);			
		    print " " x 20 . "Incorrect\n";
		    $con->Attr(0x17);
		    print " " x 20 . "The answer was: \"" .
			    $noun->{"question"} . "\" = \"" . 
			    $noun->{"answer"} . "\"\n";			
		    # see if it was the answer to something else
		    foreach (@{$self->{questions}})
		    {
			if ($_->{answer} eq $input)
			{
			     print " " x 20 . "You may be confused with : \"" .
				  $_->{"question"} . "\" = \"" .
				  $_->{"answer"} . "\"\n";
			} 
		    }
		    
	    }

	}

	print "\n";

    

    }

}



# pick the next noun
# first look for a noun that is due for repetition
# then look for a new noun
# then look for any noun
# return undef when all nouns have been quizzed at least 4 times.
sub pickNext
{
    my $self = shift;    
    my $nounObj = $self->{questions};
    my $random = rand (scalar @$nounObj);
        
    #pick two nouns, schedule the one of the two that is
    #in the lowest box
    my $pickNoun = undef;
    my $pickNoun2 = undef;
    
    # look for any nouns that are due for repetition
    my $dueNoun = undef;

    #any noun, a fallback in case there are no others
    my $anyNoun = undef;
    
    #  search all nouns for a suitable to display right now
    #
    foreach my $noun (@$nounObj[$random .. scalar @$nounObj-1], @$nounObj[0 .. $random-1])
    {
	unless ($anyNoun)
	{
	    if ($noun->{box} < $self->{bins}) { $anyNoun = $noun; }
	}
	unless ($dueNoun)
	{
	    #if it is due already for 3 turns, pick it
	    if ($self->{counter} - $noun->{howSoon} > 3 && 
	        $noun->{howSoon} != -1)
	   	{    
		    $dueNoun = $noun;    
		}
	    #elseif it is due for one turn or more, pick it 30% of the times
	    elsif ($self->{counter} - $noun->{howSoon} > 2 &&
	        $noun->{howSoon} != -1 && rand(100) < 30)
		{
		    $dueNoun = $noun;
		}    
	}
	unless ($pickNoun)
	{
	    if ($noun->{box} < $self->{bins} &&
		$noun->{howSoon} == -1) { $pickNoun = $noun; }
	}
	unless ($pickNoun2)
	{
	    if ($pickNoun && $noun->{box} < $self->{bins} &&
	        $noun->{howSoon} == -1)
	    {
	        $pickNoun2 = $noun;
	    }
	}
    }

    if ($dueNoun) { return $dueNoun; }
    if ($pickNoun2 && $pickNoun->{box} > $pickNoun2->{box})
    {
        return $pickNoun2;
    }
    if ($pickNoun) { return $pickNoun; }
    return $anyNoun;
}

1;
