#!/usr/bin/perl
#
# Dr. Higgins v0.95 
# Intelligent Vocabulary Quiz 
# to help you with your self-study of a foreign language
# 
# Copyright (C) 2004-2005 Martijn van Iersel 
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
#
#
# (planned) lesson variables
# format: #tag=value\n
#   #question1: question for one way, "" is substituted (default = "What is "")
#   #question2: question the other way, "" is substituted
#   #askbothways: 1 or 0, default = 1
#   #bins, default = 4
#   #charset, not implemented yet
# all tags are optional

use strict;
use warnings;
use Storable;
use Quiz;
use NounObj;
use MyCon;

### CONSTANTS ###
my $VERSION = "0.95";
my $settingsfile = 'settings.sto';
my $debugMode = 0;
my $logfile = 'higlog.txt';

### GLOBALS ###
my $con = MyCon->con_maker();
my $quiz = undef;
	
$con->Attr(0x17);
$con->Cls(0x17);

if (-e $settingsfile)
{
    $quiz = retrieve ($settingsfile);
    if (defined ($quiz))
    {
	print "Successfully reloaded previous settings.\n";
    }
}

sub choose_file()
{
	my $filename;
	while ()
		{

			$con->Cls(0x17);
			my $i = 1;
			my @files = glob ('lessons/*.txt');
			foreach (@files)
			{
				printf "%3d: $_\n", $i;				
				$i++;
			}
			
			print "\nSelect a file (or press enter to cancel)\n";
			my $choice = <STDIN>;
			chomp $choice;
			if ($choice eq "")
			{
				$filename = undef;
				last;
			}
			elsif ($choice > 0 && $choice <= scalar @files)
			{
				$filename = $files[$choice-1];				
				last;
			}
			else
			{
			       print "I didn't understand, please try again\n";
			}
		}
	return $filename;
}

sub menu_wordsfile()
{
    my $filename = choose_file();
    unless ($filename)
    {
	print "Action cancelled.\n";
    }
    else
    {
	$quiz = new Quiz();
	$quiz->reload ($filename);
	print "New file selected.\n";
    }
}

sub menu_quit()
{
    if ($quiz)
    {
	store $quiz, $settingsfile;
	print "Saved settings to $settingsfile\n";
    }
    $con->Attr(0x07);
    $con->Cls(0x07);

    print "Goodbye...\n";
    exit;
}

unless ($quiz)
{
    print "I couldn't load previous settings. Please select a words file\n";
    menu_wordsfile();
    unless ($quiz)
    {
	menu_quit();
    }
}

while ()
{
    $con->Cls(0x17);
    $con->Cursor (0,0);
    print "Dr. Higgins Version $VERSION (c) 2004-2005 by Martijn van Iersel



    What do you want to do?
    
    [S] Start / continue current quiz
    [R] Reset quiz
    [W] Change words file (currently $quiz->{filename})
    [B] Change number of bins (currently $quiz->{bins}) 
    [L] Print log
    [Q] Quit

";

    my $answer = <STDIN>;
    chomp $answer;

    if ($answer eq "s" || $answer eq "S")
    {
	my $starttime = time;
	my $startcounter = $quiz->{counter};
	$quiz->run($con, $debugMode);
	$con->Cls(0x17);
	if ($quiz->{counter} - $startcounter > 10)
	{
		open LOG, ">>$logfile";
		my $endtime = time;
		my $duration = $endtime - $starttime;
		printf LOG 
			"%24s #q: %4i dur.: %02i:%02i:%02i %10s\n",
			"" . localtime ($starttime),
			$quiz->{counter} - $startcounter,
			($duration / 3600) % 100,
			($duration / 60) % 60,
			$duration % 60,
			$quiz->{filename};
		close LOG;
	}		
	print "Going back to the main menu...\n";
    } 
    elsif ($answer eq "q" || $answer eq "Q")
    {
	menu_quit();
    } 
    elsif ($answer eq "r" || $answer eq "R")
    {
	my $wordsfile = $quiz->{filename};
	$quiz = new Quiz();
	$quiz->reload($wordsfile);
    } 
    elsif ($answer eq "b" || $answer eq "B")
    {   
	print "How many bins do you want?\n";
	my $bins = <STDIN>;
	chomp $bins;
	if ($bins < 1 || $bins > 10)
	{
	    print "Failed. Enter a number between 1 and 10 and try again.\n";
	}
	else
	{
	    print "Ok\n";
	    $quiz->{bins} = $bins;
	}
    } 
    elsif ($answer eq 'w' || $answer eq 'W')
    {
	menu_wordsfile();
    }
    elsif (($answer eq "l") || ($answer eq "L"))
    {
	print "\n\nLOG FILE\n\n";
	open LOG, "$logfile";
	while (my $line = <LOG>)
	{
	    print $line;
	}
	close LOG;
	print "Press ENTER to continue\n";
	<STDIN>;
    }
    else
    {
	print "I didn't understand your command, please try again\n";
    }
}


